@@ -40,7 +40,6 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <assert.h>
 #include <signal.h>
 #include <pthread.h>
 #include <errno.h>
@@ -85,6 +84,10 @@ struct wl_shm_pool {
  */
 struct wl_shm_buffer {
 	struct wl_resource *resource;
+	int internal_refcount;
+	int external_refcount;
+	struct wl_client *client;
+	struct wl_listener client_destroy_listener;
 	int32_t width, height;
 	int32_t stride;
 	uint32_t format;
@@ -144,12 +147,16 @@ shm_pool_unref(struct wl_shm_pool *pool, bool external)
 {
 	if (external) {
 		pool->external_refcount--;
-		assert(pool->external_refcount >= 0);
+		if (pool->external_refcount < 0)
+			wl_abort("Requested to unref an external reference to "
+				 "pool but none found\n");
 		if (pool->external_refcount == 0)
 			shm_pool_finish_resize(pool);
 	} else {
 		pool->internal_refcount--;
-		assert(pool->internal_refcount >= 0);
+		if (pool->internal_refcount < 0)
+			wl_abort("Requested to unref an internal reference to "
+				 "pool but none found\n");
 	}
 
 	if (pool->internal_refcount + pool->external_refcount > 0)
@@ -162,13 +169,38 @@ shm_pool_unref(struct wl_shm_pool *pool, bool external)
 	free(pool);
 }
 
+static void
+shm_buffer_unref(struct wl_shm_buffer *buffer, bool external)
+{
+	if (external) {
+		buffer->external_refcount--;
+		if (buffer->external_refcount < 0) {
+			wl_abort("Requested to unref an external reference to "
+				 "buffer but none found\n");
+		}
+	} else {
+		buffer->internal_refcount--;
+		if (buffer->internal_refcount < 0) {
+			wl_abort("Requested to unref an internal reference to "
+				 "buffer but none found\n");
+		}
+	}
+
+	if (buffer->internal_refcount + buffer->external_refcount > 0)
+		return;
+
+	if (buffer->client)
+		wl_list_remove(&buffer->client_destroy_listener.link);
+	shm_pool_unref(buffer->pool, false);
+	free(buffer);
+}
+
 static void
 destroy_buffer(struct wl_resource *resource)
 {
 	struct wl_shm_buffer *buffer = wl_resource_get_user_data(resource);
 
-	shm_pool_unref(buffer->pool, false);
-	free(buffer);
+	shm_buffer_unref(buffer, false);
 }
 
 static void
@@ -202,6 +234,17 @@ format_is_supported(struct wl_client *client, uint32_t format)
 	return false;
 }
 
+
+static void
+shm_buffer_client_destroy_notify(struct wl_listener *listener, void *data)
+{
+	struct wl_shm_buffer *buffer =
+		wl_container_of(listener, buffer, client_destroy_listener);
+
+	buffer->client = NULL;
+	wl_list_remove(&buffer->client_destroy_listener.link);
+}
+
 static void
 shm_pool_create_buffer(struct wl_client *client, struct wl_resource *resource,
 		       uint32_t id, int32_t offset,
@@ -234,6 +277,14 @@ shm_pool_create_buffer(struct wl_client *client, struct wl_resource *resource,
 		return;
 	}
 
+	buffer->client = client;
+	buffer->client_destroy_listener.notify =
+		shm_buffer_client_destroy_notify;
+	wl_client_add_destroy_listener(buffer->client,
+				       &buffer->client_destroy_listener);
+
+	buffer->internal_refcount = 1;
+	buffer->external_refcount = 0;
 	buffer->width = width;
 	buffer->height = height;
 	buffer->format = format;
@@ -441,7 +492,7 @@ wl_shm_buffer_get(struct wl_resource *resource)
 }
 
 WL_EXPORT int32_t
-wl_shm_buffer_get_stride(struct wl_shm_buffer *buffer)
+wl_shm_buffer_get_stride(const struct wl_shm_buffer *buffer)
 {
 	return buffer->stride;
 }
@@ -458,8 +509,8 @@ wl_shm_buffer_get_stride(struct wl_shm_buffer *buffer)
  * SIGBUS signals. This can happen if the client claims that the
  * buffer is larger than it is or if something truncates the
  * underlying file. To prevent this signal from causing the compositor
- * to crash you should call wl_shm_buffer_begin_access and
- * wl_shm_buffer_end_access around code that reads from the memory.
+ * to crash you should call wl_shm_buffer_begin_access() and
+ * wl_shm_buffer_end_access() around code that reads from the memory.
  *
  * \memberof wl_shm_buffer
  */
@@ -475,23 +526,62 @@ wl_shm_buffer_get_data(struct wl_shm_buffer *buffer)
 }
 
 WL_EXPORT uint32_t
-wl_shm_buffer_get_format(struct wl_shm_buffer *buffer)
+wl_shm_buffer_get_format(const struct wl_shm_buffer *buffer)
 {
 	return buffer->format;
 }
 
 WL_EXPORT int32_t
-wl_shm_buffer_get_width(struct wl_shm_buffer *buffer)
+wl_shm_buffer_get_width(const struct wl_shm_buffer *buffer)
 {
 	return buffer->width;
 }
 
 WL_EXPORT int32_t
-wl_shm_buffer_get_height(struct wl_shm_buffer *buffer)
+wl_shm_buffer_get_height(const struct wl_shm_buffer *buffer)
 {
 	return buffer->height;
 }
 
+/** Reference a shm_buffer
+ *
+ * \param buffer The buffer object
+ *
+ * Returns a pointer to the buffer and increases the refcount.
+ *
+ * The compositor must remember to call wl_shm_buffer_unref() when
+ * it no longer needs the reference to ensure proper destruction
+ * of the buffer.
+ *
+ * \memberof wl_shm_buffer
+ * \sa wl_shm_buffer_unref
+ */
+WL_EXPORT struct wl_shm_buffer *
+wl_shm_buffer_ref(struct wl_shm_buffer *buffer)
+{
+	buffer->external_refcount++;
+	return buffer;
+}
+
+/** Unreference a shm_buffer
+ *
+ * \param buffer The buffer object
+ *
+ * Drops a reference to a buffer object.
+ *
+ * This is only necessary if the compositor has explicitly
+ * taken a reference with wl_shm_buffer_ref(), otherwise
+ * the buffer will be automatically destroyed when appropriate.
+ *
+ * \memberof wl_shm_buffer
+ * \sa wl_shm_buffer_ref
+ */
+WL_EXPORT void
+wl_shm_buffer_unref(struct wl_shm_buffer *buffer)
+{
+	shm_buffer_unref(buffer, true);
+}
+
 /** Get a reference to a shm_buffer's shm_pool
  *
  * \param buffer The buffer object
@@ -499,7 +589,7 @@ wl_shm_buffer_get_height(struct wl_shm_buffer *buffer)
  * Returns a pointer to a buffer's shm_pool and increases the
  * shm_pool refcount.
  *
- * The compositor must remember to call wl_shm_pool_unref when
+ * The compositor must remember to call wl_shm_pool_unref() when
  * it no longer needs the reference to ensure proper destruction
  * of the pool.
  *
@@ -509,9 +599,6 @@ wl_shm_buffer_get_height(struct wl_shm_buffer *buffer)
 WL_EXPORT struct wl_shm_pool *
 wl_shm_buffer_ref_pool(struct wl_shm_buffer *buffer)
 {
-	assert(buffer->pool->internal_refcount +
-	       buffer->pool->external_refcount);
-
 	buffer->pool->external_refcount++;
 	return buffer->pool;
 }
@@ -613,7 +700,7 @@ init_sigbus_data_key(void)
  * In order to make the compositor robust against clients that change
  * the size of the underlying file or lie about its size, you should
  * protect access to the buffer by calling this function before
- * reading from the memory and call wl_shm_buffer_end_access
+ * reading from the memory and call wl_shm_buffer_end_access()
  * afterwards. This will install a signal handler for SIGBUS which
  * will prevent the compositor from crashing.
  *
@@ -624,15 +711,15 @@ init_sigbus_data_key(void)
  *
  * If a SIGBUS signal is received for an address within the range of
  * the SHM pool of the given buffer then the client will be sent an
- * error event when wl_shm_buffer_end_access is called. If the signal
+ * error event when wl_shm_buffer_end_access() is called. If the signal
  * is for an address outside that range then the signal handler will
  * reraise the signal which would will likely cause the compositor to
  * terminate.
  *
  * It is safe to nest calls to these functions as long as the nested
- * calls are all accessing the same buffer. The number of calls to
- * wl_shm_buffer_end_access must match the number of calls to
- * wl_shm_buffer_begin_access. These functions are thread-safe and it
+ * calls are all accessing the same pool. The number of calls to
+ * wl_shm_buffer_end_access() must match the number of calls to
+ * wl_shm_buffer_begin_access(). These functions are thread-safe and it
  * is allowed to simultaneously access different buffers or the same
  * buffer from multiple threads.
  *
@@ -658,18 +745,19 @@ wl_shm_buffer_begin_access(struct wl_shm_buffer *buffer)
 		pthread_setspecific(wl_shm_sigbus_data_key, sigbus_data);
 	}
 
-	assert(sigbus_data->current_pool == NULL ||
-	       sigbus_data->current_pool == pool);
+	if (!(sigbus_data->current_pool == NULL ||
+	       sigbus_data->current_pool == pool))
+		wl_abort("Incorrect pool passed for current thread\n");
 
 	sigbus_data->current_pool = pool;
 	sigbus_data->access_count++;
 }
 
-/** Ends the access to a buffer started by wl_shm_buffer_begin_access
+/** Ends the access to a buffer started by wl_shm_buffer_begin_access()
  *
  * \param buffer The SHM buffer
  *
- * This should be called after wl_shm_buffer_begin_access once the
+ * This should be called after wl_shm_buffer_begin_access() once the
  * buffer is no longer being accessed. If a SIGBUS signal was
  * generated in-between these two calls then the resource for the
  * given buffer will be sent an error.
@@ -686,13 +774,22 @@ wl_shm_buffer_end_access(struct wl_shm_buffer *buffer)
 		return;
 
 	sigbus_data = pthread_getspecific(wl_shm_sigbus_data_key);
-	assert(sigbus_data && sigbus_data->access_count >= 1);
+	if (!(sigbus_data && sigbus_data->access_count >= 1))
+		wl_abort("sigbus_data is NULL or wl_shm_buffer_begin_access "
+			 "wasn't called before\n");
 
 	if (--sigbus_data->access_count == 0) {
 		if (sigbus_data->fallback_mapping_used) {
-			wl_resource_post_error(buffer->resource,
-					       WL_SHM_ERROR_INVALID_FD,
-					       "error accessing SHM buffer");
+			if (buffer->resource) {
+				wl_resource_post_error(buffer->resource,
+						       WL_SHM_ERROR_INVALID_FD,
+						       "error accessing SHM buffer");
+			} else if (buffer->client) {
+				wl_client_post_implementation_error(buffer->client,
+								    "Error accessing SHM buffer of a "
+								    "wl_buffer resource which has "
+								    "already been destroyed");
+			}
 			sigbus_data->fallback_mapping_used = 0;
 		}
 
